@@ -453,7 +453,7 @@ static void process_text(mpg123_handle *fr, unsigned char *realdata, size_t real
 		if(NOQUIET) error("Unable to attach new text!");
 		return;
 	}
-	mdebug("process_text: (over)writing entry with ID %s", t->id
+	mdebug("process_text: (over)writing entry with ID %s", t->id[0]
 	?	(char[5]) { t->id[0], t->id[1], t->id[2], t->id[3], 0 }
 	:	"(nil)" );
 	memcpy(t->id, id, 4);
@@ -1214,7 +1214,7 @@ int parse_new_id3(mpg123_handle *fr, unsigned long first4bytes)
 									if(fr->rva.level[rva_mode] <= rva2+1)
 									{
 										pos += strlen((char*) realdata) + 1;
-										debug2("got my pos: %zu - %zu", realsize, pos);
+										debug2("got my pos: %lu - %lu", realsize, pos);
 										// channel and two bytes for RVA value
 										// pos possibly just past the safety zero, so one more than realsize
 										if(pos > realsize || realsize-pos < 3)
