@@ -11,13 +11,17 @@
 #define _BSD_SOURCE
 #include "config.h"
 #include "compat.h"
+#if defined(WIN32) && defined(DYNAMIC_BUILD)
+#define LINK_MPG123_DLL
+#endif
 #include "mpg123.h"
 #include "getlopt.h"
 #include <errno.h>
 #include <ctype.h>
-#include "debug.h"
 #include "win32_support.h"
 
+#include "debug.h"
+
 static int errors = 0;
 
 static struct
